import ast
from typing import Optional

from lumei.ai.openai.agent import create_agent
from lumei.ai.openai.file_search import file_search
from lumei.utils.file_manager import save_result, DataDescription, find_matched_files, setup_output_file, \
    check_if_can_create_file, create_data_description


def run_file_search_and_store_results(
        openai_api_key: str,
        input_files: str,
        output_file: str,
        file_search_query: str
) -> int:
    err = check_if_can_create_file(output_file)

    if err:
        print(err)
        return 1

    print("Verified permission to create file.")

    data_descriptions, err = convert_file_search_query_to_objects(file_search_query)

    if err:
        print(err)
        return 1

    query = {}
    for data_description in data_descriptions:
        query[data_description.name] = data_description.description

    print("Parsed list of names and data descriptions for file search.")

    files, err = find_matched_files(input_files)

    if err:
        print(err)
        return 1

    print(f"Created list of files to process. Found {len(files)} files.")

    agent, err = create_agent(openai_api_key)

    if err:
        print(err)
        return 1

    print(f"Created agent with id: {agent.id}.")

    print(f"Begun processing files.")

    results: [dict[str, str]] = []

    for file in files:
        result, error = file_search(agent, file, query)

        if result:
            print(f"    {file} SUCCESS")
            results.append(result)
        elif error:
            print(f"    {file} FAILED: {error}")
        else:
            print(f"    {file} FAILED")

    print(f"Finished processing files.")

    err = setup_output_file(output_file, data_descriptions)

    if err:
        print(err)
        return 1

    print(f"Created output file: {output_file}.")

    err = save_result(output_file, data_descriptions, results)

    if err:
        print(err)
        return 1

    print(f"Wrote result to file: {output_file}.")

    return 0


def convert_file_search_query_to_objects(file_search_query: str) -> [Optional[list[DataDescription]], Optional[str]]:
    print(file_search_query)

    try:
        queries = ast.literal_eval(file_search_query)
        descriptions: [list[DataDescription]] = []

        for query in queries:
            description, err = create_data_description(query)
            if err:
                return None, f"Failed to parse file search query: {query}"
            descriptions += [description]

        return descriptions, None
    except Exception as e:
        return None, f"Could not parse file search query json input: {e}"



import csv
import os
import re
from typing import Optional

import pandas as pd

file_search_query_example = """
[
    { "name": "Student Name", "description": "Name of the student." },
    { "name": "School", "description": "Student's school" },
    { "name": "Major", "description": "Student's major." }
]
"""


class DataDescription:
    def __init__(self, name: str, description: str):
        self.name = name
        self.description = description


def create_data_description(query: dict[str, str]) -> [Optional[DataDescription], Optional[str]]:
    name = query['name']
    description = query['description']

    if not name:
        return None, f"Missing name field for file search query. Example of correct query: {file_search_query_example}"

    if not description:
        return None, f"Missing description field for file search query. Example of correct query: {file_search_query_example}"

    return DataDescription(name, description), None


def check_if_can_create_file(output_file) -> Optional[str]:
    if not output_file.endswith((".csv", ".xlsx", ".json")):
        return "Only \".csv\", \".xlsx\", or \".json\" file types are supported."

    try:
        with open(output_file, "w"):
            pass
    except PermissionError:
        return f"Permission denied for creating output file: \"{output_file}\"."
    except Exception as e:
        return f"An error occurred: {e}"

    return None


def save_result(output_file: str, dataset: [DataDescription], results: [dict[str, str]]) -> Optional[str]:
    rows: [dict[str, str]] = []

    for result in results:
        row = [result[data.name] for data in dataset]
        rows.append(row)

    try:
        with open(output_file, "w", newline=""):
            df = pd.DataFrame(results)

            if output_file.endswith(".csv"):
                df.to_csv(output_file, index=False)
            if output_file.endswith(".xlsx"):
                df.to_excel(output_file, index=False)
            if output_file.endswith(".json"):
                df.to_json(output_file, orient='records', lines=False, indent=4)
        return None
    except Exception as e:
        return f"Failed to write result to output file {e}"


def setup_output_file(output_file: str, dataset: [DataDescription]) -> Optional[str]:
    os.makedirs(os.path.dirname(output_file), exist_ok=True)

    with open(output_file, "w", newline="") as file:
        column_headers = [data.name for data in dataset]
        writer = csv.writer(file)
        writer.writerow(column_headers)

    return None


def find_matched_files(input_string: str) -> [Optional[list[str]], Optional[str]]:
    parts = re.split(r"[,\n]+", input_string)
    cleaned_parts = [part.strip() for part in parts if part.strip()]

    files = []

    for part in cleaned_parts:
        files += find_matched_files_for_pattern(part)

    if not files:
        return None, f"No file found given locations: {input_string}"

    return files, None


def find_matched_files_for_pattern(pattern) -> [str]:
    if os.path.isdir(pattern):
        return [os.path.join(pattern, f) for f in os.listdir(pattern) if os.path.isfile(os.path.join(pattern, f))]

    if os.path.isfile(pattern):
        return [pattern]

    directory, file_pattern = os.path.split(pattern)
    if not directory:
        directory = "."

    matched_files = []
    for filename in os.listdir(directory):
        if re.match(file_pattern, filename):
            matched_files.append(os.path.join(directory, filename))

    return matched_files



import argparse
import os

from lumei.ai.openai.query import run_file_search_and_store_results
from lumei.utils.file_manager import file_search_query_example

parser = argparse.ArgumentParser(description='Lumei file search processor')

parser.add_argument(
    "--input-files",
    default=None,
    required=False,
    help="Source files to process on. "
         "Multiple files can be provided and they are seperated by a comma \",\" character. "
         "File inputs can be expressed as a path to a single file or a regex."
)
parser.add_argument(
    "--output-file",
    default=None,
    required=False,
    help="Path of the file that the results will be written to. "
         "Input must be a file path to a single file. "
         "Supported file formate are \".csv\", \".xlsx\", and \".json\"."
)
parser.add_argument(
    "--file-search-query",
    default=None,
    required=False,
    help="Name and description of data to search for. "
         "Input should be an array of JSON objects."
         "Name of the data to search for is the key. Name of the data will be the column name for the result dataset. "
         "The description of the data to search is the value."
         "\n\n"
         "Example:"
         f"{file_search_query_example}"
)
parser.add_argument(
    "--openai-api-key",
    default=None,
    required=False,
    help="API key for OpenAI, necessary for file search functionalities. "
         "Alternative way to provide the API key is to set as \"OPENAI_API_KEY\" environment variable."
)
args = parser.parse_args()


def main():
    if args.file_search_query:
        if not args.input_files:
            print("Source directory not provided.")
            exit(1)

        if not args.output_file:
            print("Destination directory not provided.")
            exit(1)

        openai_api_key = None

        if args.openai_api_key:
            openai_api_key = args.openai_api_key
        elif os.getenv("OPEN_AI_API_KEY"):
            openai_api_key = os.getenv("OPENAI_API_KEY")

        if not openai_api_key:
            print("OpenAI API key not provided.")
            exit(1)

        print("Starting file search process.")

        exit_code = run_file_search_and_store_results(
            openai_api_key=openai_api_key,
            input_files=args.input_files,
            output_file=args.output_file,
            file_search_query=args.file_search_query,
        )

        exit(exit_code)
    else:
        print("Type \"lumei --help\" for additional info.")
        exit(1)


if __name__ == '__main__':
  main()



import atexit

from setuptools import find_packages, setup

from lumei import __version__


def print_message():
  print('Type `lumei` in the CLI to start searching file content.')


atexit.register(print_message)

with open('requirements.txt') as f:
  requirements = [line for line in f.read().splitlines() if line]

with open("README.md", "r") as fh:
  long_description = fh.read()

setup(
  name='lumei',
  version=__version__,
  description='File search tool using OpenAI assistant.',
  long_description=long_description,
  long_description_content_type="text/markdown",
  url='https://github.com/hansololz/lumei',
  keywords=["lumei", 'openai', 'file search'],
  packages=find_packages(),
  include_package_data=True,
  python_requires='>=3.12',
  install_requires=requirements,
  entry_points={
    'console_scripts': [
      'lumei=lumei.main:main',
    ]
  },
  license='GPL',
)
